# TP 1 TS326

import numpy as np
import matplotlib.pyplot as plt
from utils_corr import compute_inertia, score_classif


plt.close('all')

data = np.load('./data/cloud_data_1.npy')
N = data.shape[0]
classif_true = data[:,2].astype('int')-1
K = np.unique(classif_true).shape[0]
p = 2
X = data[:,0:p].T

colors = np.array([[1, 0, 0],[0, 1, 0], [0, 0, 1], [0, 0, 0], [1, 1, 0], [1, 0, 1], [0, 1, 1]])

plt.figure(1)
plt.subplot(121)
for j in range(0,N):
    plt.plot(data[j,1], data[j,0], marker='*', color=colors[classif_true[j],:])

#%%

#% INITIALISATION %%%

iter = 5

#% Init seeds Gi = zi
Gi = np.zeros((p,K))

init = 2 #0 = random, #1 = ginput, #2 = kmeans++
use_medo = 0 #0 = barycentre, 1 = point central

if (init==0):
    for pp in range(0,p):
        Gi[pp,0:K] = np.random.rand(1,K)*(np.max(X[pp,:])-np.min(X[pp,:]))+np.min(X[pp,:])
elif (init==1):
        xy = np.array(plt.ginput(K))
        for i in range(0,K):
            Gi[0,i] = xy[i][1]
            Gi[1,i] = xy[i][0]
elif (init==2):
        #Sélection d'un échantillon aléatoire
        Gi[:, 0] = X[:, np.ceil(np.random.rand(1)*N).astype('int')].T
        #Distances D^2(x) des données X au premier Gi
        D2 = np.zeros((N,1))
        gi = Gi[:,0]
        D2 = np.sum((X - gi[:,np.newaxis])**2, axis=0)
        for i in range(1, K):
            
            H = D2/np.sum(D2)           
            Gi[:,i] = X[:,np.random.choice(range(0,N), p=H)]
            
            #Distance de chaque point X à tous les barycentres
            Dk = np.zeros((N,i+1))
            for j in range(0, i+1):
                gi = Gi[:,j]
                Dk[:,j] = np.sum((X-gi[:,np.newaxis])**2, axis=0)
        
            #Sélection de la plus petite distance
            D2 = np.min(Dk, axis=1)

plt.plot(Gi[1,:], Gi[0,:], '+c', markersize=12) 



#%% K means loop

I_inter = np.zeros((iter,1))
I_intra = np.zeros((iter,1))
I_tot = np.zeros((iter,1))
score = np.zeros((iter,1))
dist = np.zeros((N,K))
classif_exp = np.ones((N,1)) #Vecteur de classif

for i in range(0, iter):
    
    print(i)

    #Compute closest association
    for n in range(0,K):
        gi = Gi[:,n]
        dist[:,n] = np.sum((gi[:,np.newaxis] - X)**2, axis=0)
    
    classif_exp = np.nanargmin(dist, axis=1)
    
    #Update barycenters
    if (use_medo):
        #Medoids
        for n in range(0,K):
             Xn = X[:, classif_exp == n]
             dist_n = np.zeros((Xn.shape[1],1))
             for nn in range(0, Xn.shape[1]):
                 xn = Xn[:,nn]
                 dist_n[nn] = np.sum(((Xn - xn[:,np.newaxis])**2))
             
             p = np.argmin(dist_n)
             Gi[:,n] = Xn[:,p]
    else:
       #Classic average
       for n in range(0,K):
            Gi[:,n] = np.mean(data[classif_exp == n, 0:p], axis=0)
   
    #Score classif
    score[i] = score_classif(K, classif_exp, classif_true)
   
    #plot res
    plt.figure(1)
    plt.subplot(122)
    for j in range(0,N):
        plt.plot(data[j,1], data[j,0], color=colors[classif_exp[j],:], marker='*')
    for k in range(0,K):
        plt.plot(Gi[1,k], Gi[0,k], marker='+', color=colors[k,:], markersize=10+i*5) # 'MarkerSize', 10)
        plt.title('Iteration {} - score {}'.format(i, score[i]))
    plt.show()
    plt.pause(1)
   
    #Inertia
    I_inter_i, I_intra_i, I_tot_i = compute_inertia(K, Gi, data, classif_exp)
    I_inter[i] = I_inter_i
    I_intra[i] = I_intra_i
    I_tot[i] = I_tot_i
   
plt.figure(3)
plt.plot(range(0,iter), I_inter)
plt.plot(range(0,iter), I_intra)
plt.plot(range(0,iter), I_tot)
plt.legend(['Inter classes', 'Intra classes', 'Totale'])
   

